import WCInput from  '/webui/mangosteen/mangosteen/static/zwc/wc_input.js' ;
import WCHidden from  '/webui/mangosteen/mangosteen/static/zwc/wc_hidden.js' ;
import WCTab from  '/webui/mangosteen/mangosteen/static/zwc/wc_tab.js' ;

import WCTable from  '/webui/mangosteen/mangosteen/static/zwc/wc_table.js' ;
import WCCheckbox from  '/webui/mangosteen/mangosteen/static/zwc/wc_checkbox.js' ;
import WCModal from  '/webui/mangosteen/mangosteen/static/zwc/wc_modal.js' ;
import WCRadio from  '/webui/mangosteen/mangosteen/static/zwc/wc_radio.js' ;
import WCNav from  '/webui/mangosteen/mangosteen/static/zwc/wc_nav.js' ;

import WCMessage from  '/webui/mangosteen/mangosteen/static/zwc/wc_message.js' ;

import WCGroup from  '/webui/mangosteen/mangosteen/static/zwc/wc_group.js' ;
import WCForm from  '/webui/mangosteen/mangosteen/static/zwc/wc_form.js' ;

